import fileinput
import shutil
import os
import re
import glob

# Get SASHome path with error handling
sas_home = input("Enter the path to SASHome: ")

if not sas_home or not os.path.exists(sas_home):
    print("Invalid or non-existent path provided. Script cannot continue.")
    exit(1)
    

#===============================================================================================================
# START - Processing of sas.svcs.remoting jars 
print("\n\nProcessing sas.svcs.remoting jars")
pattern = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "sas.svcs.remoting_*")

# Find the directory matching the pattern
matching_dirs = glob.glob(pattern)

if matching_dirs:
    # Count the number of matching directories
    count_matching_dirs = len(matching_dirs)
    print("Number of directories matching pattern sas.svcs.remoting_*  are : ", count_matching_dirs)
    
    # Addition of httpcore to remoting_jar
    search_string = "httpcore"
    
    for remoting_dir in matching_dirs:
        remoting_jar = os.path.join(remoting_dir, "META-INF", "MANIFEST.MF")
        print("Search and replace in progress", remoting_jar)
        
        with open(remoting_jar, "r") as file_rem:
            lines = file_rem.readlines()

        start_index = -1
        end_index = -1
        in_require_bundle_section = False

        for i, line in enumerate(lines):
            if line.startswith("Require-Bundle:"):
                start_index = i
                in_require_bundle_section = True
            elif in_require_bundle_section and not line.startswith(" "):
                end_index = i
                break

        if start_index != -1 and end_index != -1:
            require_bundle_section = lines[start_index:end_index]

            if search_string not in " ".join(require_bundle_section):
                lines[end_index - 1] = lines[end_index - 1].rstrip("\n")
                lines[end_index - 1] += f",{search_string}\n"

                with open(remoting_jar, "w") as file_rem:
                    file_rem.writelines(lines)

                print(f"Added '{search_string}' to Require-Bundle.")
            else:
                print(f"'{search_string}' is already present in Require-Bundle.")
        else:
            print("No 'Require-Bundle' found in the manifest.mf file.")
print("Search and replace completed for sas.svcs.remoting jars")
# END - Processing of sas.svcs.remoting jars 

#===============================================================================================================
# START - Processing of sas.svc.webdav jars 
print("\n\nProcessing sas.svc.webdav jars")
pattern = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "sas.svc.webdav_*")

# Find the directory matching the pattern
matching_dirs = glob.glob(pattern)

if matching_dirs:
    # Count the number of matching directories
    count_matching_dirs = len(matching_dirs)
    print("Number of directories matching pattern sas.svc.webdav_*  are : ", count_matching_dirs)
    
    # Addition of httpcore to webdav_jar
    search_string = "httpcore"
    
    for webdav_dir in matching_dirs:
        webdav_jar = os.path.join(webdav_dir, "META-INF", "MANIFEST.MF")
        print("Search and replace in progress", webdav_jar)

        with open(webdav_jar, "r") as file_webdav:
            lines = file_webdav.readlines()

        start_index = -1
        end_index = -1
        in_require_bundle_section = False

        for i, line in enumerate(lines):
            if line.startswith("Require-Bundle:"):
                start_index = i
                in_require_bundle_section = True
            elif in_require_bundle_section and not line.startswith(" "):
                end_index = i
                break

        if start_index != -1 and end_index != -1:
            require_bundle_section = lines[start_index:end_index]

            if search_string not in " ".join(require_bundle_section):
                lines[end_index - 1] = lines[end_index - 1].rstrip("\n")
                lines[end_index - 1] += f",{search_string}\n"

                with open(webdav_jar, "w") as file_webdav:
                    file_webdav.writelines(lines)

                print(f"Added '{search_string}' to Require-Bundle.")
            else:
                print(f"'{search_string}' is already present in Require-Bundle.")
        else:
            print("No 'Require-Bundle' found in the manifest.mf file.")
print("Search and replace completed") 
# END - Processing of sas.svc.webdav jars 

#===============================================================================================================
# START - Processing of Poi jars 
print("\n\nProcessing Poi jars")
pattern = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "Poi_*")

# Find the directory matching the pattern
matching_dirs = glob.glob(pattern)

if matching_dirs:
    # Count the number of matching directories
    count_matching_dirs = len(matching_dirs)
    print("Number of directories matching pattern Poi_* are : ", count_matching_dirs)
    
    # Adding Log4J2 and commons_io to poi_jar
    search_strings = ["Log4J2", "commons_io"]
    
    for poi_dir in matching_dirs:
        poi_jar = os.path.join(poi_dir, "META-INF", "MANIFEST.MF")
        print("Search and replace in progress", poi_jar)

        with open(poi_jar, "r") as file_poi:
            lines = file_poi.readlines()

        start_index = -1
        end_index = -1
        in_require_bundle_section = False

        for i, line in enumerate(lines):
            if line.startswith("Require-Bundle:"):
                start_index = i
                in_require_bundle_section = True
            elif in_require_bundle_section and not line.startswith(" "):
                end_index = i
                break

        if start_index != -1 and end_index != -1:
            require_bundle_section = lines[start_index:end_index]

            for search_string in search_strings:
                if search_string not in " ".join(require_bundle_section):
                    lines[end_index - 1] = lines[end_index - 1].rstrip("\n")
                    lines[end_index - 1] += f",{search_string}\n"

                    with open(poi_jar, "w") as file_poi:
                        file_poi.writelines(lines)

                    print(f"Added '{search_string}' to Require-Bundle.")
                else:
                    print(f"'{search_string}' is already present in Require-Bundle.")
        else:
            print("No 'Require-Bundle' found in the manifest.mf file.")

print("Search and replace completed")
# END - Processing of Poi jars 


#=============================================================================================================== 
# START - Processing of jackrabbit-webdav-client jars 
print("\n\nProcessing jackrabbit-webdav-client jars")
pattern = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "jackrabbit-webdav-client_*")

# Find the directory matching the pattern
matching_dirs = glob.glob(pattern)

if matching_dirs:
    # Count the number of matching directories
    count_matching_dirs = len(matching_dirs)
    print("Number of directories matching pattern jackrabbit-webdav-client_* are : ", count_matching_dirs)
    
    # Add httpclient and httpcore to jackrabbit_jar
    search_strings = ["httpclient", "httpcore"]
    
    for jackrabbit_jar in matching_dirs:
        jackrabbit_jar = os.path.join(jackrabbit_jar, "META-INF", "MANIFEST.MF")
        print("Search and replace in progress", jackrabbit_jar)

        with open(jackrabbit_jar, "r") as file_jackrabbit:
            lines = file_jackrabbit.readlines()

        start_index = -1
        end_index = -1
        in_require_bundle_section = False

        for i, line in enumerate(lines):
            if line.startswith("Require-Bundle:"):
                start_index = i
                in_require_bundle_section = True
            elif in_require_bundle_section and not line.startswith(" "):
                end_index = i
                break

        if start_index != -1 and end_index != -1:
            require_bundle_section = lines[start_index:end_index]

            for search_string in search_strings:
                if search_string not in " ".join(require_bundle_section):
                    lines[end_index - 1] = lines[end_index - 1].rstrip("\n")
                    lines[end_index - 1] += f",{search_string}\n"

                    with open(jackrabbit_jar, "w") as file_jackrabbit:
                        file_jackrabbit.writelines(lines)

                    print(f"Added '{search_string}' to Require-Bundle.")
                else:
                    print(f"'{search_string}' is already present in Require-Bundle.")
        else:
            print("No 'Require-Bundle' found in the manifest.mf file.")
print("Search and replace completed")
# END - Processing of jackrabbit-webdav-client jars 


#=============================================================================================================== 
#START - Remove trailing blank lines from springframework_security_jar
print("\n\nProcessing org.springframework.security jars for trailing spaces")
pattern = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "org.springframework.security_*")
springframework_security_jar = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "org.springframework.security_5.7.10.0_SAS_20230726161526", "META-INF", "MANIFEST.MF")

# Find the directory matching the pattern
matching_dirs = glob.glob(pattern)

if matching_dirs:
    # Count the number of matching directories
    count_matching_dirs = len(matching_dirs)
    print("Number of directories matching pattern org.springframework.security_* are : ", count_matching_dirs)
    
    for springframework_security_jar in matching_dirs:
        springframework_security_jar = os.path.join(springframework_security_jar, "META-INF", "MANIFEST.MF")
        
        print("Removing the trailing empty lines from the file ", jackrabbit_jar)
        
        with open(springframework_security_jar, "r") as file_rem:
            lines = file_rem.readlines()

        while lines and lines[-1].strip() == "":
            lines.pop()

        content = ''.join(lines)

        with open(springframework_security_jar, "w") as file_rem:
            file_rem.write(content)
            
print("Completed removing the trailing spaces from org.springframework.security")
# END - Remove trailing blank lines from springframework_security_jar


#=============================================================================================================== 
# START - Processing of org.springframework.security jars 
print("\n\nProcessing org.springframework.security jars for manifest update")
pattern = os.path.join(sas_home, "SASVersionedJarRepository", "eclipse", "plugins", "org.springframework.security_*")
 
# Find the directory matching the pattern
matching_dirs = glob.glob(pattern)

if matching_dirs:
    # Count the number of matching directories
    count_matching_dirs = len(matching_dirs)
    print("Number of directories matching pattern org.springframework.security_* are : ", count_matching_dirs)
    
    # Adding the Required-Bundle string to springframework_security_jar
    string_to_add = "Require-Bundle: org.springframework.core"
    
    for springframework_security_jar in matching_dirs:
        springframework_security_jar = os.path.join(springframework_security_jar, "META-INF", "MANIFEST.MF")
        print("Search and replace in progress", springframework_security_jar)

        with open(springframework_security_jar, "r") as file_sec:
            content = file_sec.read()

        if string_to_add not in content:
            with open(springframework_security_jar, "a") as file_sec:
                file_sec.write(string_to_add + "\n")
            print(f"Added '{string_to_add}' to the file.")
        else:
            print(f"'{string_to_add}' is already present in the file. No changes needed.")
print("Search and replace completed")
# END - Processing of org.springframework.security jars 
#=============================================================================================================== 


# Update the plugin versions in the config.ini file of the Financial Management Studio
def update_plugins_versions(config_file, plugins_versions):
    updated_plugins = set()

    with open(config_file, 'r') as file:
        lines = file.readlines()

    updated = False

    with open(config_file, 'w') as file:
        for line in lines:
            for plugin, current_version, new_version in plugins_versions:
                if f'/{plugin}_' in line and plugin not in updated_plugins:
                    match = re.search(r'([0-9]+[._a-zA-Z0-9]*)', line)
                    if match:
                        current_version_found = match.group(0)
                        if current_version_found == current_version:
                            line = re.sub(current_version, new_version, line)
                            updated = True
                            updated_plugins.add(plugin)
                            print(f"Version for {plugin} has been updated to {new_version}.")

            file.write(line)

    if not updated:
        print("No entries found for the specified plugins in the configuration file.")

config_file = os.path.join(sas_home, "SASFinancialManagementStudio", "5.62", "configuration", "config.ini")

plugins_versions = [
    ('commons_net', '3.8.0.0_SAS_20210420102417', '3.9.0.0_SAS_20230809100246'),
    ('org.springframework.aop', '5.3.21.0_SAS_20220623103303', '5.3.29.0_SAS_20230724074727'),
    ('org.springframework.beans', '5.3.21.0_SAS_20220623103300', '5.3.29.0_SAS_20230720095434'),
    ('org.springframework.context', '5.3.21.0_SAS_20220623103305', '5.3.29.0_SAS_20230724080603'),
    ('org.springframework.context.support', '5.3.21.0_SAS_20220623103308', '5.3.29.0_SAS_20230724081252'),
    ('org.springframework.core', '5.3.21.0_SAS_20220623103257', '5.3.29.0_SAS_20230720094915'),
    ('org.springframework.expression', '5.3.21.0_SAS_20220623103300', '5.3.29.0_SAS_20230724075312'),
    ('org.springframework.jdbc', '5.3.21.0_SAS_20220623103311', '5.3.29.0_SAS_20230724092840'),
    ('org.springframework.jms', '5.3.21.0_SAS_20220623103311', '5.3.29.0_SAS_20230724093110'),
    ('org.springframework.messaging', '5.3.21.0_SAS_20220623142022', '5.3.29.0_SAS_20230724093241'),
    ('org.springframework.orm', '5.3.21.0_SAS_20220623103314', '5.3.29.0_SAS_20230724095551'),
    ('org.springframework.oxm', '5.3.21.0_SAS_20220623103308', '5.3.29.0_SAS_20230724100241'),
    ('org.springframework.security', '5.7.5.0_SAS_20221122180141', '5.7.10.0_SAS_20230726161526'),
    ('org.springframework.transaction', '5.3.21.0_SAS_20220623103308', '5.3.29.0_SAS_20230724092509'),
    ('org.springframework.web', '5.3.21.0_SAS_20220623103312', '5.3.29.0_SAS_20230724101123'),
    ('org.springframework.web.servlet', '5.3.21.0_SAS_20220623103315', '5.3.29.0_SAS_20230726083502'),
    ('org.springframework.webmvc', '5.3.21.0_SAS_20220623103315', '5.3.29.0_SAS_20230724101714')
]
print("\n\nProcessing configuration file of SAS Financial Management Studio")
update_plugins_versions(config_file, plugins_versions)
#=============================================================================================================== 
print("\nManifest changes done. If you are re-running, please ensure you clean up the manifests from the previous run if any.")